import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:shelter/widgets.dart';

import 'common_util.dart';
import 'dialog_item.dart';

export 'dialog_item.dart';

class DialogUtil {
  final BuildContext context;

  DialogUtil._(this.context);

  factory DialogUtil.of(BuildContext context) {
    return DialogUtil._(context);
  }

  AlertDialog buildDialog({
    String title,
    Widget content,
    String leftTitle,
    String rightTitle,
    VoidCallback leftCallback,
    VoidCallback rightCallback,
  }) {
    return AlertDialog(
      title: let(
        title,
        (text) => Text(text, style: Theme.of(context).textTheme.title),
      ),
      content: content,
      actions: <Widget>[
        if (leftTitle != null) ...[
          FlatButton(
            textColor: Theme.of(context).disabledColor,
            onPressed: () {
              if (leftCallback != null) {
                leftCallback();
              }
            },
            child: Text(leftTitle),
          )
        ],
        if (rightTitle != null) ...[
          FlatButton(
            textColor: Theme.of(context).buttonColor,
            onPressed: () {
              if (rightCallback != null) {
                rightCallback();
              }
            },
            child: Text(rightTitle),
          )
        ],
      ],
    );
  }

  Dialog buildLoadingDialog({@required Widget child, double width = 280.0, double height = 86.0}) {
    return Dialog(
      child: Container(
        width: width,
        height: height,
        alignment: Alignment.center,
        child: Row(
          mainAxisSize: MainAxisSize.min,
          children: [
            const SizedBox(
              child: CircularProgressIndicator(
                strokeWidth: 3.0,
              ),
              width: 25.0,
              height: 25.0,
            ),
            Padding(
              padding: const EdgeInsets.only(left: 16.0),
              child: child,
            ),
          ],
        ),
      ),
    );
  }

  Future<bool> showMessageDialog({
    bool barrierDismissible = true,
    String title,
    String message,
    String leftTitle = '취소',
    String rightTitle = '확인',
  }) async {
    final result = await showDialog<bool>(
      context: context,
      barrierDismissible: barrierDismissible,
      builder: (BuildContext context) {
        final navigator = Navigator.of(context);
        Widget current = buildDialog(
          title: title,
          content: Text(message, style: Theme.of(context).textTheme.body1),
          leftTitle: leftTitle,
          rightTitle: rightTitle,
          leftCallback: () => navigator.pop(false),
          rightCallback: () => navigator.pop(true),
        );
        if (!barrierDismissible) {
          current = WillPopScope(
            onWillPop: () async => false,
            child: current,
          );
        }
        return current;
      },
    );
    return result ?? false;
  }

  Future<void> showErrorDialog({
    String title,
    String message,
    String rightTitle = '확인',
  }) async {
    await showMessageDialog(
      barrierDismissible: false,
      title: title,
      message: message,
      leftTitle: null,
      rightTitle: rightTitle,
    );
  }

  /// Show Input dialog for String return value
  ///
  /// How to use?
  ///
  /// ```
  /// DialogUtil.showInputDialog(
  ///   context: context,
  ///   initialValue: 'Initial Value',
  ///   title: 'Please set something.',
  ///   maxLength: 30,
  ///   callback: (newValue) async {
  ///     // Do something
  ///   }
  /// );
  /// ```
  Future<String> showInputDialog({
    String initialValue,
    String title,
    String message,
    String hint,
    TextInputType keyboardType,
    bool useObscure = false,
    FormFieldValidator<String> validator,
    String leftTitle = '취소',
    String rightTitle = '확인',
    int maxLength,
  }) {
    String value = initialValue;
    final formKey = GlobalKey<FormState>();
    return showDialog(
      context: context,
      builder: (BuildContext context) {
        return buildDialog(
          title: title,
          content: SingleChildScrollView(
            child: Column(
              mainAxisSize: MainAxisSize.min,
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                if (message != null) ...[
                  Text(
                    message,
                    style: Theme.of(context).textTheme.body1,
                  ),
                ],
                Padding(
                  padding: const EdgeInsets.only(top: 16.0),
                  child: ShelterTextField(
                    formKey: formKey,
                    hint: hint,
                    initialValue: initialValue,
                    keyboardType: keyboardType,
                    useObscure: useObscure,
                    onChangeText: (text) => value = text,
                    maxLength: maxLength,
                    validator: validator,
                  ),
                ),
              ],
            ),
          ),
          leftTitle: leftTitle,
          rightTitle: rightTitle,
          leftCallback: () => Navigator.of(context).pop(),
          rightCallback: () {
            if (formKey.currentState.validate()) {
              Navigator.of(context).pop(value);
            }
          },
        );
      },
    );
  }

  Widget _buildListItem<T>(DialogItem<T> item) {
    final navigator = Navigator.of(context);
    return Container(
      color: Theme.of(context).cardColor,
      child: FlatButton(
        child: SizedBox(
          width: double.infinity,
          child: Text(
            item.displayName,
            style: Theme.of(context).textTheme.subhead,
          ),
        ),
        onPressed: () => navigator.pop(item.value),
      ),
    );
  }

  Future<T> showListDialog<T>({
    String title,
    List<DialogItem<T>> list = const [],
  }) {
    assert(list != null);
    final titleWidget = Text(
      title,
      style: Theme.of(context).textTheme.headline,
    );
    final actions = list.map((item) => _buildListItem(item)).toList();
    if (Theme.of(context).platform == TargetPlatform.iOS) {
      return showCupertinoModalPopup(
        context: context,
        builder: (_) => CupertinoActionSheet(title: titleWidget, actions: actions),
      );
    } else {
      return showDialog(
        context: context,
        builder: (_) => SimpleDialog(title: titleWidget, children: actions),
      );
    }
  }

  Future<String> showSimpleListDialog({
    String title,
    List<String> list = const [],
  }) {
    return showListDialog(
      title: title,
      list: list.map((value) => DialogItem(value, value)).toList(),
    );
  }

  Future<T> showCustomDialog<T>({
    bool barrierDismissible = true,
    Widget Function(BuildContext) builder,
  }) async {
    return await showDialog(
      context: context,
      barrierDismissible: barrierDismissible,
      builder: builder,
    );
  }
}
