import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/services.dart';
import 'package:shelter/enums.dart';
import 'package:shelter/exceptions.dart';

void _toFirebaseError(dynamic error, StackTrace stackTrace) {
  if (error is PlatformException) {
    throw FirebaseException(findFirebaseErrorTypeByCode(error.code), error.message);
  } else {
    throw error;
  }
}

Future<FirebaseUser> createUserWithEmailAndPassword(String email, String password) {
  return FirebaseAuth.instance
      .createUserWithEmailAndPassword(email: email, password: password)
      .catchError(_toFirebaseError);
}

Future<FirebaseUser> signInWithCredential(AuthCredential credential) {
  return FirebaseAuth.instance.signInWithCredential(credential).catchError(_toFirebaseError);
}

Future<FirebaseUser> signInWithEmailAndPassword(String email, String password) {
  return FirebaseAuth.instance
      .signInWithEmailAndPassword(
        email: email,
        password: password,
      )
      .catchError(_toFirebaseError);
}

Future<FirebaseUser> reauthenticateWithCredential(FirebaseUser user, AuthCredential credential) {
  return user.reauthenticateWithCredential(credential).catchError(_toFirebaseError);
}

Future<FirebaseUser> updateEmail(FirebaseUser user, String email) async {
  await user.updateEmail(email);
  return FirebaseAuth.instance.currentUser().catchError(_toFirebaseError);
}

Future<bool> sendPasswordResetEmail(String email) async {
  await FirebaseAuth.instance.sendPasswordResetEmail(email: email).catchError(_toFirebaseError);
  return true;
}

Future<bool> sendEmailVerification(FirebaseUser user) async {
  await user.sendEmailVerification().catchError(_toFirebaseError);
  return true;
}

// Refer https://github.com/flutter/flutter/issues/20390 for user.reload()
Future<bool> isEmailVerified(FirebaseUser user) async {
  await user.reload();
  final reloadedUser = await FirebaseAuth.instance.currentUser();
  return reloadedUser.isEmailVerified;
}

AuthCredential getFacebookCredential(String accessToken) {
  try {
    return FacebookAuthProvider.getCredential(accessToken: accessToken);
  } catch (error) {
    switch (error.message) {
      case 'FIRAuthErrorDomain':
        throw FirebaseException(FirebaseErrorType.unauthorized, error.message);
      default:
        throw FirebaseException(FirebaseErrorType.unknown, error.message);
    }
  }
}

AuthCredential getGoogleCredential(String idToken, String accessToken) {
  try {
    return GoogleAuthProvider.getCredential(
      idToken: idToken,
      accessToken: accessToken,
    );
  } catch (error) {
    switch (error.message) {
      case 'com.google.android.gms.common.api.ApiException: 10: ':
        throw FirebaseException(FirebaseErrorType.unauthorized, error.message);
      default:
        throw FirebaseException(FirebaseErrorType.unknown, error.message);
    }
  }
}
