import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:quiver/strings.dart';
import 'package:shelter/src/utils/common_util.dart';

class ShelterTextField extends StatefulWidget {
  const ShelterTextField({
    Key key,
    this.formKey,
    this.label,
    this.hint,
    this.initialValue,
    this.keyboardType,
    this.useObscure = false,
    this.autoCorrect = true,
    this.enabled = true,
    this.maxLength,
    this.forceErrorStream,
    this.onValidated,
    this.onChangeText,
    this.validator,
    this.iconBuilder,
    this.obscureIconBuilder,
    this.onIconPressed,
    this.onSubmitted,
  }) : super(key: key);

  final GlobalKey<FormState> formKey;
  final String label;
  final String hint;
  final String initialValue;
  final TextInputType keyboardType;
  final bool useObscure;
  final bool autoCorrect;
  final bool enabled;
  final int maxLength;
  final Stream<String> forceErrorStream;
  final ValueChanged<bool> onValidated;
  final FormFieldSetter<String> onChangeText;
  final FormFieldValidator<String> validator;
  final void Function(String) onSubmitted;

  final Icon Function(bool isValid) iconBuilder;
  final Icon Function(bool isObscure) obscureIconBuilder;
  final VoidCallback onIconPressed;

  @override
  State<StatefulWidget> createState() => _ShelterTextFieldState(formKey, useObscure);
}

class _ShelterTextFieldState extends State<ShelterTextField> {
  final GlobalKey<FormState> _formKey;
  TextEditingController _inputController;
  StreamSubscription<String> _forceErrorSubscription;
  bool _validated;
  String _errorText;
  bool _obscure;

  _ShelterTextFieldState(GlobalKey<FormState> formKey, bool useObscure)
      : _formKey = formKey ?? GlobalKey<FormState>(),
        _obscure = useObscure;

  @override
  void initState() {
    super.initState();
    if (isNotBlank(widget.initialValue)) {
      Future.microtask(_onFormChanged);
    }
    _inputController = TextEditingController(text: widget.initialValue);
    _forceErrorSubscription = widget.forceErrorStream?.listen((errorString) {
      setState(() {
        _errorText = errorString;
        _validated = false;
      });
    });
  }

  @override
  void dispose() {
    _forceErrorSubscription?.cancel();
    _inputController?.dispose();
    super.dispose();
  }

  Icon get icon {
    if (widget.iconBuilder != null) {
      return widget.iconBuilder(_validated ?? false);
    }
    if (widget.useObscure && widget.obscureIconBuilder != null) {
      return widget.obscureIconBuilder(_obscure ?? false);
    }
    return null;
  }

  VoidCallback get onIconPressed {
    if (widget.onIconPressed != null) {
      return widget.onIconPressed;
    }
    if (widget.useObscure) {
      return () {
        setState(() => _obscure = !_obscure);
      };
    }
    return null;
  }

  InputDecoration _buildTextFormDecoration() {
    final decorationTheme = Theme.of(context).inputDecorationTheme;
    return InputDecoration(
      hintText: widget.hint,
      errorText: _errorText,
      suffixIcon: let(icon, (_icon) {
        return IconButton(
          icon: icon,
          color: Theme.of(context).iconTheme.color,
          onPressed: onIconPressed,
        );
      }),
    ).applyDefaults(decorationTheme);
  }

  void _hideKeyboard() {
    FocusScope.of(context).requestFocus(FocusNode());
  }

  void _onFormChanged() {
    _formKey.currentState.save();
    setState(() {
      _errorText = null;
      _validated = _formKey.currentState.validate();
    });
    if (widget.onValidated != null) {
      widget.onValidated(_validated);
    }
  }

  @override
  Widget build(BuildContext context) {
    final bool invalidInput = _validated != null && !_validated;
    final themeData = Theme.of(context);
    Widget current = TextFormField(
      controller: _inputController,
      decoration: _buildTextFormDecoration(),
      keyboardType: widget.keyboardType,
      style: themeData.textTheme.subhead.copyWith(fontWeight: FontWeight.normal),
      obscureText: _obscure,
      autocorrect: widget.autoCorrect,
      maxLength: widget.maxLength,
      inputFormatters: [
        if (widget.maxLength != null) ...[
          TextInputFormatter.withFunction((oldValue, newValue) {
            if (newValue.text.length > widget.maxLength) {
              return oldValue;
            } else {
              return newValue;
            }
          })
        ]
      ],
      onEditingComplete: () {
        _hideKeyboard();
        widget.onSubmitted(_inputController.text);
      },
      onSaved: widget.onChangeText,
      validator: widget.validator,
      cursorColor: invalidInput ? themeData.errorColor : themeData.cursorColor,
      enabled: widget.enabled,
    );

    current = Form(
      key: _formKey,
      child: current,
      onChanged: _onFormChanged,
    );

    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[
        if (widget.label != null) ...[
          Text(
            widget.label,
            style: themeData.textTheme.subtitle.copyWith(
              color: ifValueOrNull(invalidInput, themeData.errorColor),
            ),
          ),
          const SizedBox(height: 8.0),
        ],
        current,
      ],
    );
  }
}
